package io.hmit.modules.fhjw.controller;

import io.hmit.common.annotation.LogOperation;
import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.ExcelUtils;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.fhjw.dto.FhiwFunctionDTO;
import io.hmit.modules.fhjw.excel.FhiwFunctionExcel;
import io.hmit.modules.fhjw.service.FhiwFunctionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 评价模块
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2021-06-22
 */
@RestController
@RequestMapping("fhjw/fhiwfunction")
@Api(tags="评价模块")
public class FhiwFunctionController {
    @Autowired
    private FhiwFunctionService fhiwFunctionService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
    @RequiresPermissions("fhjw:fhiwfunction:page")
    public Result<PageData<FhiwFunctionDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<FhiwFunctionDTO> page = fhiwFunctionService.page(params);

        return new Result<PageData<FhiwFunctionDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @RequiresPermissions("fhjw:fhiwfunction:info")
    public Result<FhiwFunctionDTO> get(@PathVariable("id") Long id){
        FhiwFunctionDTO data = fhiwFunctionService.get(id);

        return new Result<FhiwFunctionDTO>().ok(data);
    }

    @GetMapping("/getAllFuncList")
    @ApiOperation("获取所有的评价模版")
    public Result<List<FhiwFunctionDTO>> getAllFuncList(){
        List<FhiwFunctionDTO> data = fhiwFunctionService.getAllFuncList();

        return new Result<List<FhiwFunctionDTO>>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @RequiresPermissions("fhjw:fhiwfunction:save")
    public Result save(@RequestBody FhiwFunctionDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        fhiwFunctionService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @RequiresPermissions("fhjw:fhiwfunction:update")
    public Result update(@RequestBody FhiwFunctionDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        fhiwFunctionService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @RequiresPermissions("fhjw:fhiwfunction:delete")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        fhiwFunctionService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @RequiresPermissions("fhjw:fhiwfunction:export")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<FhiwFunctionDTO> list = fhiwFunctionService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, FhiwFunctionExcel.class);
    }

}
