package io.hmit.modules.fhjw.controller;

import io.hmit.common.annotation.LogOperation;
import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.ExcelUtils;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.fhjw.dto.FhjwCommentDTO;
import io.hmit.modules.fhjw.excel.FhjwCommentExcel;
import io.hmit.modules.fhjw.service.FhjwCommentService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 案件评价表
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2021-06-02
 */
@RestController
@RequestMapping("fhjw/fhjwcomment")
@Api(tags="案件评价表")
public class FhjwCommentController {
    @Autowired
    private FhjwCommentService fhjwCommentService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
//    @RequiresPermissions("fhjw:fhjwcomment:page")
    public Result<PageData<FhjwCommentDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<FhjwCommentDTO> page = fhjwCommentService.page(params);

        return new Result<PageData<FhjwCommentDTO>>().ok(page);
    }

    @GetMapping("/getAnswer")
    @ApiOperation("根据caseId和userId获取评价问题")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "caseId", value = "案件ID", paramType = "query", required = true, dataType="int") ,
            @ApiImplicitParam(name = "userId", value = "用户ID", paramType = "query",required = true, dataType="int")
    })

    public Result<FhjwCommentDTO> get(@ApiIgnore @RequestParam Map<String, Object> params){
        FhjwCommentDTO data = fhjwCommentService.assembleDTO(params);

        return new Result<FhjwCommentDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @RequiresPermissions("fhjw:fhjwcomment:save")
    public Result save(@RequestBody FhjwCommentDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        fhjwCommentService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @RequiresPermissions("fhjw:fhjwcomment:update")
    public Result update(@RequestBody FhjwCommentDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        fhjwCommentService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @RequiresPermissions("fhjw:fhjwcomment:delete")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        fhjwCommentService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @RequiresPermissions("fhjw:fhjwcomment:export")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<FhjwCommentDTO> list = fhjwCommentService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, FhjwCommentExcel.class);
    }

}
