package io.hmit.modules.fhjw.controller;

import io.hmit.common.annotation.LogOperation;
import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.ExcelUtils;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.fhjw.dto.FhjwRoleFunctionDTO;
import io.hmit.modules.fhjw.excel.FhjwRoleFunctionExcel;
import io.hmit.modules.fhjw.service.FhjwRoleFunctionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 用户评价权限功能
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2021-06-22
 */
@RestController
@RequestMapping("fhjw/fhjwrolefunction")
@Api(tags="用户评价权限功能")
public class FhjwRoleFunctionController {
    @Autowired
    private FhjwRoleFunctionService fhjwRoleFunctionService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
    @RequiresPermissions("fhjw:fhjwrolefunction:page")
    public Result<PageData<FhjwRoleFunctionDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<FhjwRoleFunctionDTO> page = fhjwRoleFunctionService.page(params);

        return new Result<PageData<FhjwRoleFunctionDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @RequiresPermissions("fhjw:fhjwrolefunction:info")
    public Result<FhjwRoleFunctionDTO> get(@PathVariable("id") Long id){
        FhjwRoleFunctionDTO data = fhjwRoleFunctionService.get(id);

        return new Result<FhjwRoleFunctionDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @RequiresPermissions("fhjw:fhjwrolefunction:save")
    public Result save(@RequestBody FhjwRoleFunctionDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        fhjwRoleFunctionService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @RequiresPermissions("fhjw:fhjwrolefunction:update")
    public Result update(@RequestBody FhjwRoleFunctionDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        fhjwRoleFunctionService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @RequiresPermissions("fhjw:fhjwrolefunction:delete")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        fhjwRoleFunctionService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @RequiresPermissions("fhjw:fhjwrolefunction:export")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<FhjwRoleFunctionDTO> list = fhjwRoleFunctionService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, FhjwRoleFunctionExcel.class);
    }

}