package io.hmit.modules.fhjw.controller;

import io.hmit.common.annotation.LogOperation;
import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.ExcelUtils;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.fhjw.dto.FhjwTempDTO;
import io.hmit.modules.fhjw.dto.RoleDTO;
import io.hmit.modules.fhjw.excel.FhjwTempExcel;
import io.hmit.modules.fhjw.service.FhjwTempService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 问卷模版表
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2021-06-02
 */
@RestController
@RequestMapping("fhjw/fhjwtemp")
@Api(tags="问卷模版表")
public class FhjwTempController {
    @Autowired
    private FhjwTempService fhjwTempService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
    @RequiresPermissions("fhjw:fhjwtemp:page")
    public Result<PageData<FhjwTempDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<FhjwTempDTO> page = fhjwTempService.page(params);

        return new Result<PageData<FhjwTempDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    @RequiresPermissions("fhjw:fhjwtemp:info")
    public Result<FhjwTempDTO> get(@PathVariable("id") Long id){
        FhjwTempDTO data = fhjwTempService.get(id);

        return new Result<FhjwTempDTO>().ok(data);
    }

    @GetMapping("/getAllTemp")
    @ApiOperation("获取所有模版")
    public Result<List<FhjwTempDTO>> list() {

        List<FhjwTempDTO> list = fhjwTempService.getAllTempList();

        return new Result<List<FhjwTempDTO>>().ok(list);
    }


    @PostMapping
    @ApiOperation("保存")
    @LogOperation("保存")
    @RequiresPermissions("fhjw:fhjwtemp:save")
    public Result save(@RequestBody FhjwTempDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        fhjwTempService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    @LogOperation("修改")
    @RequiresPermissions("fhjw:fhjwtemp:update")
    public Result update(@RequestBody FhjwTempDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        fhjwTempService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    @LogOperation("删除")
    @RequiresPermissions("fhjw:fhjwtemp:delete")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        fhjwTempService.delete(ids);

        return new Result();
    }

    @GetMapping("export")
    @ApiOperation("导出")
    @LogOperation("导出")
    @RequiresPermissions("fhjw:fhjwtemp:export")
    public void export(@ApiIgnore @RequestParam Map<String, Object> params, HttpServletResponse response) throws Exception {
        List<FhjwTempDTO> list = fhjwTempService.list(params);

        ExcelUtils.exportExcelToTarget(response, null, list, FhjwTempExcel.class);
    }

}
