package io.hmit.modules.fhjw.controller;

import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.fhjw.dto.FhjwCaseDTO;
import io.hmit.modules.fhjw.service.FhjwCaseService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.ibatis.annotations.Delete;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 案件表
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2021-06-02
 */
@RestController
@RequestMapping("api/fhjwcase")
@Api(tags="案件表")
public class FhjwCaseController {
    @Autowired
    private FhjwCaseService fhjwCaseService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
    public Result<PageData<FhjwCaseDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<FhjwCaseDTO> page = fhjwCaseService.page(params);

        return new Result<PageData<FhjwCaseDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    public Result<FhjwCaseDTO> get(@PathVariable("id") Long id){
        FhjwCaseDTO data = fhjwCaseService.get(id);

        return new Result<FhjwCaseDTO>().ok(data);
    }

    @Delete("系统变更不使用")
    @GetMapping("/getBycaseNum/{caseNum}")
    @ApiOperation("根据案件号caseNum获取案件信息")
    public Result<FhjwCaseDTO> getByCaseNum(@PathVariable("caseNum") String caseNum){
        //系统信息变更
        FhjwCaseDTO data = fhjwCaseService.getByCaseNum(caseNum);

        return new Result<FhjwCaseDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    public Result save(@RequestBody Map<String, Object> params){
        //校验数据
//        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        fhjwCaseService.save(params);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    public Result update(@RequestBody FhjwCaseDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        fhjwCaseService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        fhjwCaseService.delete(ids);

        return new Result();
    }
}
