package io.hmit.modules.fhjw.controller;

import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.fhjw.dto.FhjwQuestionDTO;
import io.hmit.modules.fhjw.service.FhjwQuestionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 * 问卷问题表
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2021-06-02
 */
@RestController
@RequestMapping("api/fhjwquestion")
@Api(tags="问卷问题表")
public class FhjwQuestionController {
    @Autowired
    private FhjwQuestionService fhjwQuestionService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
    public Result<PageData<FhjwQuestionDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<FhjwQuestionDTO> page = fhjwQuestionService.page(params);

        return new Result<PageData<FhjwQuestionDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    public Result<FhjwQuestionDTO> get(@PathVariable("id") Long id){
        FhjwQuestionDTO data = fhjwQuestionService.get(id);

        return new Result<FhjwQuestionDTO>().ok(data);
    }

    @GetMapping("/getQuestionList/{tempId}")
    @ApiOperation("获取问题列表")
    public Result<List<FhjwQuestionDTO>> getQuestionList(@PathVariable("tempId") Long tempId){
        List<FhjwQuestionDTO> data = fhjwQuestionService.getQuestionList(tempId);

        return new Result<List<FhjwQuestionDTO>>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    public Result save(@RequestBody FhjwQuestionDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        fhjwQuestionService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    public Result update(@RequestBody FhjwQuestionDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        fhjwQuestionService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        fhjwQuestionService.delete(ids);

        return new Result();
    }

}
