package io.hmit.modules.user.controller;

import io.hmit.common.constant.Constant;
import io.hmit.common.page.PageData;
import io.hmit.common.utils.Result;
import io.hmit.common.validator.AssertUtils;
import io.hmit.common.validator.ValidatorUtils;
import io.hmit.common.validator.group.AddGroup;
import io.hmit.common.validator.group.DefaultGroup;
import io.hmit.common.validator.group.UpdateGroup;
import io.hmit.modules.user.dto.UserRoleDTO;
import io.hmit.modules.user.service.UserRoleService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.servlet.http.HttpServletResponse;
import java.util.List;
import java.util.Map;


/**
 * 用户权限对应表
 *
 * @author zsh 408538940@qq.com
 * @since 1.0.0 2021-06-01
 */
@RestController
@RequestMapping("api/userrole")
@Api(tags="用户权限对应表")
public class UserRoleController {
    @Autowired
    private UserRoleService userRoleService;

    @GetMapping("page")
    @ApiOperation("分页")
    @ApiImplicitParams({
        @ApiImplicitParam(name = Constant.PAGE, value = "当前页码，从1开始", paramType = "query", required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.LIMIT, value = "每页显示记录数", paramType = "query",required = true, dataType="int") ,
        @ApiImplicitParam(name = Constant.ORDER_FIELD, value = "排序字段", paramType = "query", dataType="String") ,
        @ApiImplicitParam(name = Constant.ORDER, value = "排序方式，可选值(asc、desc)", paramType = "query", dataType="String")
    })
    public Result<PageData<UserRoleDTO>> page(@ApiIgnore @RequestParam Map<String, Object> params){
        PageData<UserRoleDTO> page = userRoleService.page(params);

        return new Result<PageData<UserRoleDTO>>().ok(page);
    }

    @GetMapping("{id}")
    @ApiOperation("信息")
    public Result<UserRoleDTO> get(@PathVariable("id") Long id){
        UserRoleDTO data = userRoleService.get(id);

        return new Result<UserRoleDTO>().ok(data);
    }

    @GetMapping("/getRoleByUser/{userId}")
    @ApiOperation("信息")
    public Result<UserRoleDTO> getRoleByUser(@PathVariable("userId") Long userId){
        UserRoleDTO data = userRoleService.getRoleByUser(userId);

        return new Result<UserRoleDTO>().ok(data);
    }

    @PostMapping
    @ApiOperation("保存")
    public Result save(@RequestBody UserRoleDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, AddGroup.class, DefaultGroup.class);

        userRoleService.save(dto);

        return new Result();
    }

    @PutMapping
    @ApiOperation("修改")
    public Result update(@RequestBody UserRoleDTO dto){
        //校验数据
        ValidatorUtils.validateEntity(dto, UpdateGroup.class, DefaultGroup.class);

        userRoleService.update(dto);

        return new Result();
    }

    @DeleteMapping
    @ApiOperation("删除")
    public Result delete(@RequestBody Long[] ids){
        //校验数据
        AssertUtils.isArrayEmpty(ids, "id");

        userRoleService.delete(ids);

        return new Result();
    }
}
